#include "Display.h"


#define DISPLAY_SEGMENT_MIN (DISPLAY_PIN_NUMBER_MIN)
#define DISPLAY_SEGMENTS	(8)
#define DISPLAY_DIGIT_MIN	(DISPLAY_SEGMENT_MIN+DISPLAY_SEGMENTS)

#define DISPLAY_SEGMENT_MASK (0xFF << DISPLAY_SEGMENT_MIN)
#define DISPLAY_DIGIT_MASK (0xF << DISPLAY_DIGIT_MIN)
#define DISPLAY_MASK (DISPLAY_SEGMENT_MASK | DISPLAY_DIGIT_MASK)

static const uint8_t charset[] = { 0xFC, 0x60, 0xDA, 0xF2, 0x66,
		                           0xB6, 0xBE, 0xE0, 0xFE, 0xF6
};

#define CHARSET_SIZE (sizeof(charset))
#define CHARSET_BASE '0'

#define DECIMAL_POINT 1


void display_init(DisplayState *state) {
	// Enable GPIO Peripheral clock
	  RCC_AHBPeriphClockCmd(DISPLAY_RCC_MASKx(DISPLAY_PORT_NUMBER), ENABLE);

	  GPIO_InitTypeDef GPIO_InitStructure;

	  // Configure pin in output push/pull mode
	  GPIO_InitStructure.GPIO_Pin = DISPLAY_SEGMENT_MASK;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(DISPLAY_GPIOx(DISPLAY_PORT_NUMBER), &GPIO_InitStructure);

	  // Configure pin in output push/pull mode
	  GPIO_InitStructure.GPIO_Pin = DISPLAY_DIGIT_MASK;
	  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
	  GPIO_InitStructure.GPIO_OType = GPIO_OType_OD;
	  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
	  GPIO_Init(DISPLAY_GPIOx(DISPLAY_PORT_NUMBER), &GPIO_InitStructure);

	  display_clear(state);
}

void display_show(DisplayState *state, uint16_t number, uint16_t decimal_point) {
	int i;
	for(i=DISPLAY_DIGITS-1; i>=0; i--) {
		uint16_t digit = number % 10;
		number /= 10;
		state->segments[i] = charset[digit];
	}
	state->segments[DISPLAY_DIGITS-1-decimal_point] |= DECIMAL_POINT;
}

void display_clear(DisplayState *state) {
	state->last_digit=0;
	int i;
	for(i=0; i<DISPLAY_DIGITS; i++)
		state->segments[i]=0;
}

void display_refresh(DisplayState *state) {
	state->last_digit++;
	if (state->last_digit>=DISPLAY_DIGITS)
		state->last_digit=0;
	uint16_t segment=(state->segments[state->last_digit])<<DISPLAY_SEGMENT_MIN;
	uint16_t digit_selection= ~(1<<(state->last_digit+DISPLAY_DIGIT_MIN))
			                    & DISPLAY_DIGIT_MASK;
	uint16_t set_mask=segment | digit_selection;
	uint16_t reset_mask = ~set_mask & DISPLAY_MASK;
	if (set_mask!=0)
		GPIO_SetBits(DISPLAY_GPIOx(DISPLAY_PORT_NUMBER), set_mask);
	if (reset_mask!=0)
		GPIO_ResetBits(DISPLAY_GPIOx(DISPLAY_PORT_NUMBER), reset_mask);

}
