//
// This file is part of the GNU ARM Eclipse distribution.
// Copyright (c) 2014 Liviu Ionescu.
//

// ----------------------------------------------------------------------------

#include <stdio.h>
#include <stdint.h>



#include "Timer.h"
#include "BlinkLed.h"
#include "Button.h"
#include "Analog.h"
#include "Display.h"

#define TRUE 1
#define FALSE 0

// ----------------------------------------------------------------------------
//
// Standalone STM32F3 led blink sample (trace via DEBUG).
//
// In debug configurations, demonstrate how to print a greeting message
// on the trace device. In release configurations the message is
// simply discarded.
//
// Then demonstrates how to blink a led with 1Hz, using a
// continuous loop and SysTick delays.
//
// Trace support is enabled by adding the TRACE macro definition.
// By default the trace messages are forwarded to the DEBUG output,
// but can be rerouted to any device or completely suppressed, by
// changing the definitions required in system/src/diag/trace_impl.c
// (currently OS_USE_TRACE_ITM, OS_USE_TRACE_SEMIHOSTING_DEBUG/_STDOUT).
//
// The external clock frequency is specified as a preprocessor definition
// passed to the compiler via a command line option (see the 'C/C++ General' ->
// 'Paths and Symbols' -> the 'Symbols' tab, if you want to change it).
// The value selected during project creation was HSE_VALUE=8000000.
//
// Note: The default clock settings take the user defined HSE_VALUE and try
// to reach the maximum possible system clock. For the default 8MHz input
// the result is guaranteed, but for other values it might not be possible,
// so please adjust the PLL settings in system/src/cmsis/system_stm32f30x.c
//

// ----- Timing definitions -------------------------------------------------


#define LOOP_DELAY 1

#define VOLTAGE_MAX 3300
#define ADC_SAMPLE_MAX 4095
#define DECIMAL_DIGITS 3


// ----- main() ---------------------------------------------------------------

// Sample pragmas to cope with warnings. Please note the related line at
// the end of this function, used to pop the compiler diagnostics status.
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wunused-parameter"
#pragma GCC diagnostic ignored "-Wmissing-declarations"
#pragma GCC diagnostic ignored "-Wreturn-type"

int
main(int argc, char* argv[])
{
  // Send a greeting to the trace device (skipped on Release).
  //trace_puts("Hello ARM World!");

  // At this stage the system clock should have already been configured
  // at high speed.
  //trace_printf("System clock: %uHz\n", SystemCoreClock);

  ButtonDebouncingState debouncing_state;
  DisplayState display_state;

  timer_start();

  blink_led_init();
  button_init();
  button_init_debouncing_state(&debouncing_state);
  
  analog_init();

  display_init(&display_state);

  int prev_button=FALSE;
  int update=FALSE;


  // Infinite loop
  while (1)
    {

	  int button=button_read_debounced(&debouncing_state);
	  if (button && !prev_button)
		  update=!update;
	  prev_button=button;

	  uint16_t voltage=analog_read();
	  uint16_t value=(voltage>>9) + 1;

	  int i;
	  for(i=0; i<value; i++)
		  blink_led_on(i);
	  for(i=value; i<BLINK_PINS; i++)
		  blink_led_off(i);

	  if (update) {
		  uint16_t number=(uint32_t)voltage*VOLTAGE_MAX/ADC_SAMPLE_MAX;
		  display_show(&display_state, number, DECIMAL_DIGITS);
	  }

	  display_refresh(&display_state);

	  timer_sleep(LOOP_DELAY);
    }
  // Infinite loop, never return.
}

#pragma GCC diagnostic pop

// ----------------------------------------------------------------------------
